/**
 * @(#)AListSortInsertion.java  11 November 2006
 *
 * @author Viera K. Proulx
 */
import java.util.*;
import edu.neu.ccs.util.*;
/**
* A class that implements the 
* <CODE>{@link ASortAlgo ASortAlgo}</CODE> interface
* by implementing the insertion sort algorithm on a recursively
* define <CODE>{@link AList AList}</Code>.
*/
public class AListSortInsertion <T> extends ASortAlgo<T>{
  /** the data to be sorted saved as <CODE>{@link AList AList}</Code>. */
  AList<T> source = new MTList<T>();
    
  /**
   * The constructor initializes the 
   * <CODE>{@link Comparator Comparator}</Code>, the name of the algorithm,
   * and generates the data to be sorted from the given 
   * <CODE>{@link Traversal Traversal}</Code>
   * @param comp the <CODE>{@link Comparator Comparator}</Code>
   * that determines the ordering
   * @param tr the <CODE>{@link Traversal Traversal}</Code>
   * that generates the data to be sorted
   */
  public AListSortInsertion(Comparator<T> comp, Traversal<T> tr){
    this(comp);
    this.source = this.makeList(tr);    
  }
 
  /**
   * <P>The constructor initializes the 
   * <CODE>{@link Comparator Comparator}</Code>, 
   * the name of the algorithm.</P>
   * 
   * <P>The data to be sorted will be initialized later.</P>
   * 
   * @param comp the <CODE>{@link Comparator Comparator}</Code>
   * that determines the ordering
   */
  AListSortInsertion(Comparator<T> comp){ 
    this.comp = comp;
    this.algoName = "Insertion Sort with recursively built list";
  }
  
 
  /**
   * Initialize the data to sort with the data generated by the 
   * <CODE>{@link Traversal Traversal}</Code>.
   * 
   * @param tr the given <CODE>{@link Traversal Traversal}</Code>
   */ 
  public void initData(Traversal<T> tr){
    this.source = this.makeList(tr);
  } 
  
   /**
   * Invoke the insertion sort. Produce a
   * <CODE>{@link Traversal Traversal}</Code> to generate the result.
   * 
   * @return a * <CODE>{@link Traversal Traversal}</Code> for the sorted data.
   */
  public Traversal<T> sort(){
    return insertionSort(source);
  }
  
  /**
   * Insertion sort for recursively defined <CODE>{@link AList AList}</Code>.
   * 
   * @param tr a * <CODE>{@link Traversal Traversal}</Code>
   * for the input data
   * @return a * <CODE>{@link Traversal Traversal}</Code> for the sorted data.
   */ 
  public AList<T> insertionSort(AList<T> tr){  
    if (!tr.isEmpty()){
      return insert(tr.getFirst(), 
                    insertionSort((AList<T>)tr.getRest()));
    }
    else
      return new MTList<T>();
  }
  
  /**
   * Helper method to insert an element into a recursively defined sorted
   * <CODE>{@link AList AList}</Code>.
   * 
   * @param tr a * <CODE>{@link Traversal Traversal}</Code> for the sorted 
   * <CODE>{@link AList AList}</Code>
   * for the input data
   * @return an * <CODE>{@link AList AList}</Code> for the sorted data.
   */ 
  public AList<T> insert(T t, Traversal<T> tr){
    if (!tr.isEmpty()){
      if (comp.compare(t, tr.getFirst()) <= 0)
        return new ConsList<T>(t, (AList<T>)tr);
      else{
        return new ConsList<T>(tr.getFirst(), insert(t, tr.getRest()));
      }
    }
    else
      return new ConsList<T>(t, (AList<T>)tr);
  }
  
  /**
   * Helper method to produce a new recursively defined sorted
   * <CODE>{@link AList AList}</Code> from the data generated 
   * by a <CODE>{@link Traversal Traversal}</Code>.
   * 
   * @param tr a * <CODE>{@link Traversal Traversal}</Code> for the input data
   * @return the new data as an * <CODE>{@link AList AList}</Code> 
   */ 
  public AList<T> makeList(Traversal<T> tr){
    if (!tr.isEmpty())
      return new ConsList<T>(tr.getFirst(), makeList(tr.getRest()));
    else
      return new MTList<T>();
  }  
  
//test insertion sort
  public static void testSort(Tests tests){
    
    tests.resetData();
    
    ASortAlgo<City> insertionSortListSource = new AListSortInsertion<City>(tests.byName, tests.source);
    
    tests.tth.test("insertion sort cons list", tests.sorted, 
        insertionSortListSource.sort());

    ASortAlgo<City> insertionSortListLongunsorted = 
      new AListSortInsertion<City>(tests.byName, tests.longunsorted);

    tests.tth.test("insertion sort cons list", tests.longsorted, 
        insertionSortListLongunsorted.sort());
  }
}
