/**
 * @(#)ASortAlgo  11 November 2006
 *
 * @author Viera K. Proulx
 */

import java.util.Comparator;

/**
 * <P>Abstract class to represent an implementation of a sorting
 * algorithm that consumes the data generated by a 
 * <CODE>{@link Traversal Traversal}</CODE> and determines the
 * ordering using the <CODE>{@link Comparator Comparator}</CODE>.</P>
 * 
 * <P> The access to the result of the sort is provided by a new 
 * <CODE>{@link Traversal Traversal}</CODE>.
 */
abstract class ASortAlgo<T> {
  
  /** the <CODE>{@link Comparator Comparator}</CODE> that determines 
   * thr ordering for the sorting algorithm.
   */
  public Comparator<T> comp;
  
  /** the name of this algorithms - for reporting purposes */
  public String algoName;
  
  /**
   * Initialize the data set to be used by the algorithm 
   * with the data generated by the <CODE>{@link Traversal Traversal}</CODE>.
   */ 
  abstract public void initData(Traversal<T> tr);
  
  /**
   * <P>Sort the data set with respect to the given 
   * <CODE>{@link Comparator Comparator}</CODE>.</P>
   * <P>Produce a <CODE>{@link Traversal Traversal}</CODE> 
   * for the sorted data.</P>
   * 
   * @return the <CODE>{@link Traversal Traversal}</CODE> for the 
   * sorted data.
   */ 
  abstract public Traversal<T> sort();
  
}