/**
 * @(#)DataSet  1.0  13 November 2006
 *
 * @author Viera K. Proulx
 */

import java.util.ArrayList;
/**
 * An <CODE>{@link ArrayList ArrayList}</CODE> that holds one dataset
 * for the sorting algorithms timing tests. Each dataset records its 
 * size and whether the data has been selected in a sequential or random 
 * order.
 */
class DataSet{
  /** the actual size of the data set */
  int size;          
  /** data selected in a sequential or a random order */
  String type;        
  /** the actual city data */
  ArrayList<City> data; 
  
  /**
   * The constructor that only reserves the space, but does not
   * initalize the dataset.
   * 
   * @param size the size of this dataset
   * @param type the type of this dataset
   */
  DataSet(int size, String type){
    // reserve enough space for the expected size
    this.data = new ArrayList<City>(size);
    // but set the actual size to zero
    this.size = 0;    // should grow to the given size
    // how is this data set constructed - include in the results
    this.type = type;
  }
 
  /**
   * The constructor that sets the dataset to the given
   * <CODE>{@link ArrayList ArrayList}</CODE> of data and sets its type of
   * data selection (sequential or random).
   * 
   * @param data the <CODE>{@link ArrayList ArrayList}</CODE> data that 
   * becomes the value of this dataset
   * @param type the type of this dataset
   */
  DataSet(String type, ArrayList<City> data){
    // use the given data
    this.data = data;
    // set the actual size
    this.size = data.size();    
    // how is this data set constructed - include in the results
    this.type = type;
  }
 
  /**
   * Add the given <CODE>{@link City City}</CODE> to this data set.
   * 
   * @param city the given <CODE>{@link City City}</CODE>
   */ 
  void add(City city){
    this.data.add(city);
    this.size = this.size + 1;
  }
  
  /**
   * Provide a <CODE>{@link Traversal Traversal}</CODE> for this data set.
   * 
   * @return the desired <CODE>{@link Traversal Traversal}</CODE>
   */ 
  Traversal<City> getTraversal(){
    return new TraversalALT<City>(data);
  }
  
  /**
   * Tests for this class.
   * @param tests the <CODE>{@link Tests Tests}</CODE> instance that runs
   * this test
   */
  public static void testDataSet(Tests tests){
   City plymouth2 = new City();
    
    DataSet dset = new DataSet(4, "random");
    
    dset.add(tests.northville);
    dset.add(tests.pinckney);
    dset.add(tests.plymouth);
    dset.add(plymouth2);
    
    Traversal tr = dset.getTraversal();
    
    ArrayList<City> arrlist = new ArrayList<City>(4);
    arrlist.add(tests.northville);
    arrlist.add(tests.pinckney);
    arrlist.add(tests.plymouth);
    arrlist.add(plymouth2);
    
    DataSet dset2 = new DataSet("sequential", arrlist);
   
    tests.tth.test("Test DataSet class: data add ", 
                   tr, new TraversalALT<City>(arrlist));
    tests.tth.test("Test DataSet class: size ", dset.size, 4);
    tests.tth.test("Test DataSet class: type ", dset.type, "random");
    tests.tth.test("Test DataSet class: getTraversal ", 
                   dset2.getTraversal(), new TraversalALT<City>(arrlist));
    tests.tth.test("Test DataSet class: size ", dset2.size, 4);
    tests.tth.test("Test DataSet class: type ", dset2.type, "sequential");
    
  }
}
