/**
 * @(#)Result  13 November 2006
 *
 * @author Viera K. Proulx
 */
/**
 * A class that represents the result of one run of the timer tests
 */
public class Result{
  /** the size of the dataset that was sorted */
  int size;
  
  /** the type of the dataset that was sorted */
  String type;
  
  /** the name of the algorithm used to perform the sorting */
  String algoName;
  
  /** the name of the comparator that determined the ordering */
  String compName;
  
  /** the measured time needed to perform the sort */
  long runtime;
  
  /**
   * The constructor.
   * 
   * @param size the size of the dataset that was sorted
   * @param type the type of the dataset that was sorted
   * @param algoName the name of the algorithm used to perform the sorting
   * @param compName the name of the 
   * <CODE>Comparator Comparator</CODE> that determined the ordering
   * @param runtime the measured time needed to perform the sort
   */
  public Result(int size,
                String type,
                String algoName,
                String compName,
                long runtime){
    this.size = size;
    this.type = type;
    this.algoName = algoName;
    this.compName = compName;
    this.runtime = runtime;
  }
  
  /**
   * Represent the result as a well formatted 
   * <CODE>{@link String String}</CODE>
   * 
   * @return the well formatted <CODE>{@link String String}</CODE>
   */ 
  public String toString(){
    return "(^" + algoName + "^ ^" + compName + "^ " + size + " ^" + 
           type + "^ " + runtime + ")";
  }
  
  /**
   * Self test
   * @param argv unused
   */
  public static void main(String argv[]){
    Result r1 = new Result(8000,
                          "sequential",
                          "By Zip",
                          "Quick Sort with recursively built list",
                          1345);

    Result r2 = new Result(16000,
        "random",
        "Insertion Sort with recursively built list",
        "By Name",
        1345);
    System.out.println(r1.toString());
    System.out.println(r2.toString());
  }
}