/**
 * @(#)TestData  1.0  11 November 2006
 *
 * @author Viera K. Proulx
 */

import java.util.ArrayList;
import java.util.NoSuchElementException;

import edu.neu.ccs.console.ConsoleAware;
import edu.neu.ccs.util.MathUtilities;

/**
 * <P>This class contains the test data for all sorting tests.</P>
 * <P>For each data set size: 1000, 2000, 4000, 8000, 16000
 * there is a sequential and random set.</P>
 * <P>For size 29470 there is just one complete set.</P>
 * 
 * <P>Each sequential data set contains a contigous subset 
 * of the source data.</P>
 * <P>Each random data set selects the data at random 
 * from the original source data.</P>
 */
class TestData implements ConsoleAware{
  
  /** the original data set of all cities available */
  ArrayList<City> alldata = new ArrayList<City>();
  
  /** the 15 data sets for tests */
  ArrayList<DataSet> datasets = new ArrayList<DataSet>(15);
  
  /** 
   * <P>The constructor reads in the data file from the user-defined
   * file. It then constructs all data sets that will be used f
   * or the tests.</P>
   * <P>This assures that all tests run with the same data.</P>
   *
   */
  TestData(){
    for(Traversal<City>tr = new TraversalInFile();
        !tr.isEmpty();
        tr = tr.getRest()){
      alldata.add(tr.getFirst());
    }
    System.out.println("read initial data: " + alldata.size());
    buildDataSets();
  }
  
  /**
   * Method that initializes the datasets 
   * <CODE>{@link ArrayList ArrayList}</CODE> with the 
   * <CODE>{@link City City}</CODE> to sort.
   */
  public void buildDataSets(){
    datasets.add(new DataSet("sequential", 
        this.buildSequentialData(1000)));
    datasets.add(new DataSet("random", 
        this.buildRandomData(1000)));
    datasets.add(new DataSet("sequential", 
        this.buildSequentialData(2000)));
    datasets.add(new DataSet("random", 
        this.buildRandomData(2000)));
    datasets.add(new DataSet("sequential", 
        this.buildSequentialData(4000)));
    datasets.add(new DataSet("random", 
        this.buildRandomData(4000)));
    datasets.add(new DataSet("random", 
        this.buildSequentialData(8000)));
    datasets.add(new DataSet("sequential", 
        this.buildRandomData(8000)));
    datasets.add(new DataSet("random", 
        this.buildSequentialData(16000)));
    datasets.add(new DataSet("sequential", 
        this.buildRandomData(16000)));
    datasets.add(new DataSet("random", 
        this.buildSequentialData(29460)));
    datasets.add(new DataSet("sequential", 
        this.buildSequentialData(29460)));
    
  }
  
  /**
   * Create the base <CODE>{@link ArrayList ArrayList}</CODE> of data 
   * to use for all test datasets.
   */ 
  public void readData(){
    Traversal<City> tr = new TraversalInFile();
    this.alldata = new ArrayList<City>(30000);
    for(;
        !tr.isEmpty();
        tr.getRest()){
      alldata.add(tr.getFirst());       
    }
    console.out.println("readData produced " + alldata.size() + " elements.");
  }
  
  /**
   * Produce a random dataset of thre given size from original data.
   * 
   * @param size the desired size of the result data
   * @return the <CODE>{@link ArrayList ArrayList}</CODE> of the desired size 
   * that contains the desired source data 
   */ 
  public ArrayList<City> buildRandomData(int size){
    // make a copy of the original arraylist
    ArrayList<City> temp = (ArrayList<City>)alldata.clone();
    ArrayList<City> result = new ArrayList<City>();
    
    if (size < temp.size()){
      
      // selecting elements at random index to add to the data set
      int index = 0;
      int loc;
      
      while (index < size){
        loc = MathUtilities.randomInt(0, temp.size() - 1);
        result.add(temp.get(loc));
        index = index + 1;
      }
    }
    console.out.println("Random data set of size " + 
        result.size() + " initialized.");
    return result;
  }
  
  /**
   * Produce a sequential dataset of thre given size from original data.
   * 
   * @param size the desired size of the result data
   * @return the <CODE>{@link ArrayList ArrayList}</CODE> of the desired size 
   * that contains the desired source data 
   */ 
  public ArrayList<City> buildSequentialData(int size){
    
    // reserve space for the new  arraylist
    ArrayList<City> result = new ArrayList<City>(size);
    
    if (size < this.alldata.size()){
      
      // select at random the starting index for the data set
      int index = 0;
      int loc = MathUtilities.randomInt(0, this.alldata.size() - size);
      
      while (index < size){
        result.add(this.alldata.get(index + loc));
        index = index + 1;
      }
      console.out.println("Random data set of size " + 
          result.size() + " initialized.");
      return result;
    }
    else
      throw new NoSuchElementException(
          "data set of size " + size + "cannot be initialized");
  }
  
  
  /**
   * Verifying that the dataset sizes are as expected.
   * @param argv unused
   */
  public static void main(String[] argv){
    TestData testdata = new TestData();
    System.out.println("alldata size:  " + testdata.alldata.size());
    System.out.println("datasets size: " + testdata.datasets.size());
  }
}