/**
 * @(#)Tests  1.0  13 November 2006
 *
 * @author Viera K. Proulx
 */
import java.util.Comparator;
/**
 * Class that contains tests for all classes in this project.
 * It initializes the needed data and invokes the self tests 
 * in the static methods defined by each class.
 */
public class Tests extends TraversalTestHarness{
  
  /** Sample <CODE>{@link City City}</CODE> data */ 
  City nyc = new City();
  City boston = new City(02115, "Boston", "MA", 78.123345, 40.987654);
  City bos = new City(this.boston);
  
  City northville = new City(48167, "Northville", "MI", 83.479355, 42.426245);
  City pinckney = new City(48169, "Pinckney", "MI", 83.909918, 42.459579);
  City plymouth = new City(48170, "Plymouth", "MI", 83.479946, 42.36882);
  City gibraltar = new City(48173, "Gibraltar", "MI", 83.216196, 42.07918);
  City romulus = new City(48174, "Romulus", "MI", 83.358288, 42.220304);
  City saline = new City(48176, "Saline", "MI", 83.784936, 42.169844);
  City duluth1 = new City(55806, "Duluth", "MN", 92.127871, 46.771457);
  City duluth2 = new City(55807, "Duluth", "MN", 92.169821, 46.740783);
  City duluth3 = new City(55808, "Duluth", "MN", 92.22261, 46.68100);
  
  /** 
   * Sample <CODE>{@link AList AList}</CODE> lists of
   * <CODE>{@link City City}</CODE>.
   */ 
  AList<City> mtcities = new MTList<City>();
  AList<City> cities1 = new ConsList<City>(this.northville,
      new ConsList<City>(this.plymouth,
          new ConsList<City>(this.duluth1,
              this.mtcities)));
  AList<City> cities2 = new ConsList<City>(this.northville,
      new ConsList<City>(this.plymouth,
          new ConsList<City>(this.duluth1,
              this.mtcities)));
  
  AList<City> list1 = this.cities1;
  AList<City> list2 = this.cities2;
  
  AList<City> empty = new MTList<City>();
  
  AList<City> source = new ConsList<City>(duluth1,
      new ConsList<City>(northville,
          new ConsList<City>(plymouth, empty)));

  AList<City> sorted = new ConsList<City>(duluth1,
      new ConsList<City>(northville,
          new ConsList<City>(plymouth, empty)));

  AList<City> longunsorted = new ConsList<City>(northville,
      new ConsList<City>(plymouth,
          new ConsList<City>(romulus,
              new ConsList<City>(duluth1,
                  new ConsList<City>(gibraltar,
                      new ConsList<City>(saline,
                          new ConsList<City>(duluth2, empty)))))));

 
  AList<City> longsorted = new ConsList<City>(duluth1,
      new ConsList<City>(duluth2,
          new ConsList<City>(gibraltar,
              new ConsList<City>(northville,
                  new ConsList<City>(plymouth,
                      new ConsList<City>(romulus,
                          new ConsList<City>(saline, empty)))))));
      
  /**
   * Reset all test data to the original values.
   */
  public void resetData(){
    mtcities = new MTList<City>();
    cities1 = new ConsList<City>(this.northville,
        new ConsList<City>(this.plymouth,
            new ConsList<City>(this.duluth1,
                this.mtcities)));
    cities2 = new ConsList<City>(this.northville,
        new ConsList<City>(this.plymouth,
            new ConsList<City>(this.duluth1,
                this.mtcities)));
    
    list1 = this.cities1;
    list2 = this.cities2;
    
    empty = new MTList<City>();
    
    source = new ConsList<City>(duluth1,
        new ConsList<City>(northville,
            new ConsList<City>(plymouth, empty)));

    sorted = new ConsList<City>(duluth1,
        new ConsList<City>(northville,
            new ConsList<City>(plymouth, empty)));

    longunsorted = new ConsList<City>(northville,
        new ConsList<City>(plymouth,
            new ConsList<City>(romulus,
                new ConsList<City>(duluth1,
                    new ConsList<City>(gibraltar,
                        new ConsList<City>(saline,
                            new ConsList<City>(duluth2, empty)))))));

   
    longsorted = new ConsList<City>(duluth1,
        new ConsList<City>(duluth2,
            new ConsList<City>(gibraltar,
                new ConsList<City>(northville,
                    new ConsList<City>(plymouth,
                        new ConsList<City>(romulus,
                            new ConsList<City>(saline, empty)))))));
        
  }
  
  /** an instance of the 
   * <CODE>{@link ComparatorByName ComparatorByName}</CODE> 
   */
  public Comparator<City> byName = new ComparatorByName();   
  
  /** the test harness to run all tests */
  TraversalTestHarness tth = new TraversalTestHarness();


  /**
   * The method that invokes the static tests in all classes.
   */
  public void runTests(){
    // used in the constructor for the City
    test("Zip code converter", 12345, this.boston.decimal("12345"));
    test("Zip code converter", 2145, this.boston.decimal("02145"));
    
    // City comparison
    test("Same City", false, this.nyc.same(this.boston));   
    
    // test the test harness
    test("Lists compare", this.cities1, this.cities1);
    
    // test the test harness
    test("Traversal compare", this.list1, this.list1);
    
    AListSortInsertion.testSort(this);
    DataSet.testDataSet(this);
    TraversalALT.testTraversalALT(this);
    
    testReport();
    fullTestReport();
  }
  
  public static void main(String argv[]){
    Tests tt = new Tests();
    tt.runTests();
  }
}