/**
 * @(#)TraversalALC 11 November 2006
 *
 * @author Viera K. Proulx
 */

import java.util.ArrayList;
import java.util.NoSuchElementException;

/**
 * A non-mutable iterator for a list of elements based on 
 * <CODE>{@link ArrayList ArrayList}</CODE>
 */
public class TraversalALT<T> implements Traversal<T>{
  ArrayList<T> data;
  int index;
  
  /**
   * A constructor that starts traversing the given data 
   * at the given index. To be invoked by <CODE>getRest</CODE> method.
   * 
   * @param data the <CODE>{@link ArrayList ArrayList}</CODE> to traverse
   * @param index the index at which the traversal starts
   */   
  protected TraversalALT(ArrayList<T> data, int index){
    if (index >= 0 && index <= data.size()){
    this.data = data;
    this.index = index;
    }
    else
      throw new IndexOutOfBoundsException("In TraversalALT constructor");
  }
  
  /**
   * A constructor that starts traversing the given data 
   * at the beginning of the given <CODE>{@link ArrayList ArrayList}</CODE>.
   * 
   * @param data the <CODE>{@link ArrayList ArrayList}</CODE> to traverse
   */   
  public TraversalALT(ArrayList<T> data){
    this(data, 0);
  }
  
  /**
   * If possible, advance to the next element of the 
   * <CODE>{@link ArrayList ArrayList}</CODE> data. Throw 
   * <CODE>{@link NoSuchElementException NoSuchElementException}</CODE> 
   * if no more data is available.
   * 
   * @throws NoSuchElementException if no data is available
   */
  public Traversal<T> getRest(){
    if (this.isEmpty())
      throw new NoSuchElementException(
      "Cannot advance beyond the last element in ArrayList");
    else
      return new TraversalALT<T>(this.data, this.index + 1);
  }
  
  /**
   * If possible, produce the first element of the 
   * <CODE>{@link ArrayList ArrayList}</CODE> data. Throw 
   * <CODE>{@link NoSuchElementException NoSuchElementException}</CODE> 
   * if no more data is available.
   * 
   * @return first element of the 
   * <CODE>{@link ArrayList ArrayList}</CODE> data
   * @throws NoSuchElementException if no data is available
   */
  public T getFirst(){
    if (this.isEmpty())
      throw new NoSuchElementException(
      "No current element after the last element in ArrayList");
    else
      return this.data.get(this.index);      
  }
  
  /**
   *  Determine whether there any items left to traverse.
   *  
   *  @return true if the dataset has no more elements
   */
  public boolean isEmpty(){
    return this.index >= data.size();
  }
  
  /**
   * Tests for this class.
   * @param tests the <CODE>{@link Tests Tests}</CODE> instance that runs
   * this test
   */
  public static void testTraversalALT(Tests tests){
   City plymouth2 = new City();
    
   ArrayList<City> empty = new ArrayList<City>();
   ArrayList<City> alist = new ArrayList<City>();
   alist.add(tests.northville);
   alist.add(tests.pinckney);
   alist.add(tests.plymouth);
   alist.add(plymouth2);
    
    ArrayList<City> arrlist = new ArrayList<City>(4);
    arrlist.add(tests.northville);
    arrlist.add(tests.pinckney);
    arrlist.add(tests.plymouth);
    arrlist.add(plymouth2);
    
    Traversal<City> tr = new TraversalALT<City>(alist);
    Traversal<City> tr2 = new TraversalALT<City>(alist, 1);
     
    tests.tth.test("Test TraversalALT class: getFirst ", 
        tr.getFirst(), tests.northville);
    tests.tth.test("Test TraversalALT class: getRest ", 
        tr.getRest(), tr2);
    tests.tth.test("Test TraversalALT class: getRest ", 
        tr.getRest().getFirst(), tests.pinckney);
    tests.tth.test("Test TraversalALT class: getFirst ", 
        tr.isEmpty(), false);
    tests.tth.test("Test TraversalALT class: getFirst ", 
        tr.getRest().getRest().getRest().getRest().isEmpty(), true);
  }
}

